@@ -96,25 +96,25 @@ xyplot(expit(pred) + expit(low) + expit(upp) ~ time | sex, data = plot_data,
        type = "l", lty = c(1, 2, 2), col = c(2, 1, 1), lwd = 2,
        xlab = "Follow-up time", ylab = "Subject-Specific Probabilities")
 
-## ---- fig.align = "center", fig.width = 8.5, fig.height = 7.5, eval = FALSE----
-#  plot_data_m <- effectPlotData(fm, nDF, marginal = TRUE)
-#  
-#  # we put the two groups in the same panel
-#  my.panel.bands <- function(x, y, upper, lower, fill, col, subscripts, ..., font,
-#                             fontface) {
-#      upper <- upper[subscripts]
-#      lower <- lower[subscripts]
-#      panel.polygon(c(x, rev(x)), c(upper, rev(lower)), col = fill, border = FALSE, ...)
-#  }
-#  
-#  xyplot(expit(pred) ~ time, group = sex, data = plot_data_m,
-#         upper = expit(plot_data_m$upp), low = expit(plot_data_m$low),
-#         type = "l", col = c("blue", "red"),
-#         fill = c("#0000FF80", "#FF000080"),
-#         panel = function (x, y, ...) {
-#             panel.superpose(x, y, panel.groups = my.panel.bands, ...)
-#             panel.xyplot(x, y, lwd = 2,  ...)
-#  }, xlab = "Follow-up time", ylab = "Marginal Probabilities")
+## ---- fig.align = "center", fig.width = 8.5, fig.height = 7.5, eval = TRUE----
+plot_data_m <- effectPlotData(fm, nDF, marginal = TRUE, cores = 2)
+
+# we put the two groups in the same panel
+my.panel.bands <- function(x, y, upper, lower, fill, col, subscripts, ..., font, 
+                           fontface) {
+    upper <- upper[subscripts]
+    lower <- lower[subscripts]
+    panel.polygon(c(x, rev(x)), c(upper, rev(lower)), col = fill, border = FALSE, ...)
+}
+
+xyplot(expit(pred) ~ time, group = sex, data = plot_data_m, 
+       upper = expit(plot_data_m$upp), low = expit(plot_data_m$low), 
+       type = "l", col = c("blue", "red"), 
+       fill = c("#0000FF80", "#FF000080"),
+       panel = function (x, y, ...) {
+           panel.superpose(x, y, panel.groups = my.panel.bands, ...)
+           panel.xyplot(x, y, lwd = 2,  ...)
+}, xlab = "Follow-up time", ylab = "Marginal Probabilities")
 
 ## ---- fig.align = "center", fig.width = 8.5, fig.height = 7.5, message = FALSE----
 library("effects")
@@ -123,15 +123,11 @@ plot(predictorEffect("time", fm), type = "link")
 ## ---- fig.align = "center", fig.width = 8.5, fig.height = 7.5, message = FALSE----
 plot(predictorEffect("time", fm), type = "response")
 
-## ---- eval = FALSE-------------------------------------------------------
-#  gm <- mixed_model(fixed = y ~ sex * time, random = ~ time || id, data = DF,
-#                    family = binomial())
-#  
-#  anova(gm, fm)
-#  #>
-#  #>       AIC    BIC log.Lik  LRT df p.value
-#  #> gm 730.94 746.57 -359.47
-#  #> fm 731.66 731.66 -358.83 1.29  1  0.2564
+## ---- eval = TRUE--------------------------------------------------------
+gm <- mixed_model(fixed = y ~ sex * time, random = ~ time || id, data = DF,
+                  family = binomial())
+
+anova(gm, fm)
 
 ## ------------------------------------------------------------------------
 pred_DF <- DF[DF$id == 1, ][1:4, ]
